package com.bureau.behavioralbiometrics.data

import android.app.Application
import com.bureau.behavioralbiometrics.applifecycleevents.models.AppLifecycleEventLocal
import com.bureau.behavioralbiometrics.data.local.BBLocalDataSource
import com.bureau.behavioralbiometrics.data.remote.BBRemoteDataSource
import com.bureau.behavioralbiometrics.data.remote.models.RegisterSubSessionRequestApiModel
import com.bureau.behavioralbiometrics.di.BBScope
import com.bureau.behavioralbiometrics.di.CRED_ID
import com.bureau.behavioralbiometrics.focuschangedata.models.FocusChangeDataLocal
import com.bureau.behavioralbiometrics.frameratecollection.models.FrameRateEventLocal
import com.bureau.behavioralbiometrics.keypressTypedata.models.InputChangeEventLocal
import com.bureau.behavioralbiometrics.keypressTypedata.models.KeyPressEventLocal
import com.bureau.behavioralbiometrics.sensordata.models.AccelerometerDataLocal
import com.bureau.behavioralbiometrics.sensordata.models.GyroscopeDataLocal
import com.bureau.behavioralbiometrics.touchtypedata.models.TouchTypeDataLocal
import com.bureau.behavioralbiometrics.touchtypedata.models.ViewPortScrollDataLocal
import kotlinx.coroutines.flow.Flow
import javax.inject.Inject
import javax.inject.Named

@BBScope
class BBRepositoryImpl @Inject constructor(
    private val bbLocalDataSource: BBLocalDataSource,
    private val application: Application,
    private val bbRemoteDataSource: BBRemoteDataSource,
    @Named(CRED_ID) private val credId: String
) : BBRepository {

    override suspend fun insertAppLifecycleEvent(appLifecycleEventLocal: AppLifecycleEventLocal) {
        bbLocalDataSource.insertAppLifecycleEvent(appLifecycleEventLocal)
    }

    override fun getAllAppLifecycleEventsFlow(): Flow<List<AppLifecycleEventLocal>> {
        return bbLocalDataSource.getAllAppLifecycleEventsFlow()
    }

    override suspend fun deleteAppLifecycleEventList(appLifecycleEventLocals: List<AppLifecycleEventLocal>) {
        bbLocalDataSource.deleteAppLifecycleEventList(appLifecycleEventLocals)
    }

    override suspend fun deleteAllAppLifecycleEvents() {
        bbLocalDataSource.deleteAllAppLifecycleEvents()
    }

    override suspend fun insertTouchTypeData(touchTypeDataLocal: TouchTypeDataLocal) {
        bbLocalDataSource.insertTouchTypeData(touchTypeDataLocal)
    }

    override fun getAllTouchTypeDataFlow(): Flow<List<TouchTypeDataLocal>> {
        return bbLocalDataSource.getAllTouchTypeDataFlow()
    }

    override suspend fun deleteTouchTypeDataList(touchTypeDatumLocals: List<TouchTypeDataLocal>) {
        bbLocalDataSource.deleteTouchTypeDataList(touchTypeDatumLocals)
    }

    override suspend fun deleteAllTouchTypeData() {
        bbLocalDataSource.deleteAllTouchTypeData()
    }

    override suspend fun insertKeyPressEvent(keyPressEventLocal: KeyPressEventLocal) {
        bbLocalDataSource.insertKeyPressEvent(keyPressEventLocal)
    }

    override fun getAllKeyPressEventFlow(): Flow<List<KeyPressEventLocal>> {
        return bbLocalDataSource.getAllKeyPressEventFlow()
    }

    override suspend fun deleteKeyPressEventList(keyPressEventLocal: List<KeyPressEventLocal>) {
        bbLocalDataSource.deleteKeyPressEventList(keyPressEventLocal)
    }

    override suspend fun deleteAllKeyPressEvents() {
        bbLocalDataSource.deleteAllKeyPressEvents()
    }

    override suspend fun insertInputChangeEvent(inputChangeEventLocal: InputChangeEventLocal) {
        bbLocalDataSource.insertInputChangeEvent(inputChangeEventLocal)
    }

    override fun getAllInputChangeEventFlow(): Flow<List<InputChangeEventLocal>> {
        return bbLocalDataSource.getAllInputChangeEventFlow()
    }

    override suspend fun deleteInputChangeEventList(inputChangeEventLocal: List<InputChangeEventLocal>) {
        bbLocalDataSource.deleteInputChangeEventList(inputChangeEventLocal)
    }

    override suspend fun deleteAllInputChangeEvents() {
        bbLocalDataSource.deleteAllInputChangeEvents()
    }

    override suspend fun insertFocusChangeData(focusChangeDataLocal: FocusChangeDataLocal) {
        bbLocalDataSource.insertFocusChangeData(focusChangeDataLocal)
    }

    override fun getAllFocusChangeDataFlow(): Flow<List<FocusChangeDataLocal>> {
        return bbLocalDataSource.getAllFocusChangeDataFlow()
    }

    override suspend fun deleteFocusChangeDataList(focusChangeDatumLocals: List<FocusChangeDataLocal>) {
        bbLocalDataSource.deleteFocusChangeDataList(focusChangeDatumLocals)
    }

    override suspend fun deleteAllFocusChangeData() {
        bbLocalDataSource.deleteAllFocusChangeData()
    }

    override suspend fun insertViewPortScrollData(viewPortScrollDataLocal: ViewPortScrollDataLocal) {
        bbLocalDataSource.insertViewPortScrollData(viewPortScrollDataLocal)
    }

    override fun getAllViewPortScrollDataFlow(): Flow<List<ViewPortScrollDataLocal>> {
        return bbLocalDataSource.getAllViewPortScrollDataFlow()
    }

    override suspend fun deleteViewPortScrollDataList(viewPortScrollDatumLocals: List<ViewPortScrollDataLocal>) {
        bbLocalDataSource.deleteViewPortScrollDataList(viewPortScrollDatumLocals)
    }

    override suspend fun deleteAllViewPortScrollData() {
        bbLocalDataSource.deleteAllViewPortScrollData()
    }

    override suspend fun insertFrameRateEvent(frameRateEventLocal: FrameRateEventLocal) {
        bbLocalDataSource.insertFrameRateEvent(frameRateEventLocal)
    }

    override fun getAllFrameRateEventFlow(): Flow<List<FrameRateEventLocal>> {
        return bbLocalDataSource.getAllFrameRateEventFlow()
    }

    override suspend fun deleteFrameRateEventList(frameRateEventLocal: List<FrameRateEventLocal>) {
        bbLocalDataSource.deleteFrameRateEventList(frameRateEventLocal)
    }

    override suspend fun deleteAllFrameRateEvents() {
        bbLocalDataSource.getAllFrameRateEventFlow()
    }

    override suspend fun insertAccelerometerData(accelerometerDataLocal: AccelerometerDataLocal) {
        bbLocalDataSource.insertAccelerometerData(accelerometerDataLocal)
    }

    override fun getAllAccelerometerDataFlow(): Flow<List<AccelerometerDataLocal>> {
        return bbLocalDataSource.getAllAccelerometerDataFlow()
    }

    override suspend fun deleteAccelerometerDataList(accelerometerDataLocal: List<AccelerometerDataLocal>) {
        bbLocalDataSource.deleteAccelerometerDataList(accelerometerDataLocal)
    }

    override suspend fun deleteAllAccelerometerData() {
        bbLocalDataSource.deleteAllAccelerometerData()
    }

    override suspend fun insertGyroscopeData(gyroscopeDataLocal: GyroscopeDataLocal) {
        bbLocalDataSource.insertGyroscopeData(gyroscopeDataLocal)
    }

    override fun getAllGyroscopeDataFlow(): Flow<List<GyroscopeDataLocal>> {
        return bbLocalDataSource.getAllGyroscopeDataFlow()
    }

    override suspend fun deleteGyroscopeDataList(gyroscopeDataLocal: List<GyroscopeDataLocal>) {
        bbLocalDataSource.deleteGyroscopeDataList(gyroscopeDataLocal = gyroscopeDataLocal)
    }

    override suspend fun deleteAllGyroscopeData() {
        bbLocalDataSource.deleteAllGyroscopeData()
    }

    override suspend fun registerSubSession(
        sessionId: String,
        subSessionId: String,
        userId: String
    ): Boolean {
        val packageName = application.packageName
        val requestApiModel = RegisterSubSessionRequestApiModel(
            sessionId,
            subSessionId,
            packageName,
            userId
        )

        return bbRemoteDataSource.registerSubSession(requestApiModel, credId)
    }
}
