package com.bureau.behavioralbiometrics.data.local.dao

import androidx.room.Dao
import androidx.room.Delete
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.Query
import com.bureau.behavioralbiometrics.applifecycleevents.models.AppLifecycleEventLocal
import com.bureau.behavioralbiometrics.focuschangedata.models.FocusChangeDataLocal
import com.bureau.behavioralbiometrics.frameratecollection.models.FrameRateEventLocal
import com.bureau.behavioralbiometrics.keypressTypedata.models.InputChangeEventLocal
import com.bureau.behavioralbiometrics.keypressTypedata.models.KeyPressEventLocal
import com.bureau.behavioralbiometrics.sensordata.models.AccelerometerDataLocal
import com.bureau.behavioralbiometrics.sensordata.models.GyroscopeDataLocal
import com.bureau.behavioralbiometrics.touchtypedata.models.TouchTypeDataLocal
import com.bureau.behavioralbiometrics.touchtypedata.models.ViewPortScrollDataLocal
import kotlinx.coroutines.flow.Flow

@Dao
interface BBDao {

    @Query("SELECT * FROM app_life_cycle_events")
    fun getAllAppLifecycleEventsFlow(): Flow<List<AppLifecycleEventLocal>>

    @Insert
    suspend fun insertAllAppLifecycleEvents(appLifecycleEventLocals: List<AppLifecycleEventLocal>)

    @Delete
    suspend fun deleteAppLifecycleEvent(appLifecycleEventLocal: AppLifecycleEventLocal)

    @Delete
    suspend fun deleteAppLifecycleEventList(appLifecycleEventLocals: List<AppLifecycleEventLocal>)

    @Query("DELETE FROM app_life_cycle_events")
    suspend fun deleteAllAppLifecycleEvents()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAppLifecycleEvent(appLifecycleEventLocal: AppLifecycleEventLocal)

    @Query("SELECT * FROM touch_type_data")
    fun getAllTouchTypeDataFlow(): Flow<List<TouchTypeDataLocal>>

    @Insert
    suspend fun insertAllTouchTypeData(touchTypeDataLocalList: List<TouchTypeDataLocal>)

    @Delete
    suspend fun deleteTouchTypeData(touchTypeDataLocal: TouchTypeDataLocal)

    @Delete
    suspend fun deleteTouchTypeDataList(touchTypeDatumLocals: List<TouchTypeDataLocal>)

    @Query("DELETE FROM touch_type_data")
    suspend fun deleteAllTouchTypeData()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertTouchTypeData(touchTypeDataLocal: TouchTypeDataLocal)

    @Query("SELECT * FROM key_press_data_table")
    fun getAllKeyPressEventFlow(): Flow<List<KeyPressEventLocal>>

    @Insert
    suspend fun insertAllKeyPressEvent(keyPressEventLocalList: List<KeyPressEventLocal>)

    @Delete
    suspend fun deleteKeyPressEvent(keyPressEventLocal: KeyPressEventLocal)

    @Delete
    suspend fun deleteKeyPressEventList(keyPressEventLocal: List<KeyPressEventLocal>)

    @Query("DELETE FROM key_press_data_table")
    suspend fun deleteAllKeyPressEvent()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertKeyPressEvent(keyPressEventLocal: KeyPressEventLocal)

    @Query("SELECT * FROM input_change_data_table")
    fun getAllInputChangeEventFlow(): Flow<List<InputChangeEventLocal>>

    @Insert
    suspend fun insertAllInputChangeEvent(inputChangeEventLocalList: List<InputChangeEventLocal>)

    @Delete
    suspend fun deleteInputChangeEvent(inputChangeEventLocal: InputChangeEventLocal)

    @Delete
    suspend fun deleteInputChangeEventList(inputChangeEventLocal: List<InputChangeEventLocal>)

    @Query("DELETE FROM input_change_data_table")
    suspend fun deleteAllInputChangeEvent()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertInputChangeEvent(inputChangeEventLocal: InputChangeEventLocal)

    @Query("SELECT * FROM focus_change_data_table")
    fun getAllFocusChangeDataFlow(): Flow<List<FocusChangeDataLocal>>

    @Insert
    suspend fun insertAllFocusChangeData(focusChangeDataLocalList: List<FocusChangeDataLocal>)

    @Delete
    suspend fun deleteFocusChangeData(focusChangeDataLocal: FocusChangeDataLocal)

    @Delete
    suspend fun deleteFocusChangeDataList(focusChangeDatumLocals: List<FocusChangeDataLocal>)

    @Query("DELETE FROM focus_change_data_table")
    suspend fun deleteAllFocusChangeData()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertFocusChangeData(focusChangeDataLocal: FocusChangeDataLocal)

    @Query("SELECT * FROM view_port_scroll_data_table")
    fun getAllViewPortScrollDataFlow(): Flow<List<ViewPortScrollDataLocal>>

    @Insert
    suspend fun insertAllViewPortScrollData(viewPortScrollDataLocalList: List<ViewPortScrollDataLocal>)

    @Delete
    suspend fun deleteViewPortScrollData(viewPortScrollDataLocal: ViewPortScrollDataLocal)

    @Delete
    suspend fun deleteViewPortScrollDataList(viewPortScrollDatumLocals: List<ViewPortScrollDataLocal>)

    @Query("DELETE FROM view_port_scroll_data_table")
    suspend fun deleteAllViewPortScrollData()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertViewPortScrollData(viewPortScrollDataLocal: ViewPortScrollDataLocal)

    @Query("SELECT * FROM frame_rate_event_table")
    fun getAllFrameRateEventFlow(): Flow<List<FrameRateEventLocal>>

    @Insert
    suspend fun insertAllFrameRateEvent(frameRateEventLocalList: List<FrameRateEventLocal>)

    @Delete
    suspend fun deleteFrameRateEvent(frameRateEventLocal: FrameRateEventLocal)

    @Delete
    suspend fun deleteFrameRateEventList(frameRateEventLocal: List<FrameRateEventLocal>)

    @Query("DELETE FROM frame_rate_event_table")
    suspend fun deleteAllFrameRateEvent()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertFrameRateEvent(frameRateEventLocal: FrameRateEventLocal)

    @Query("SELECT * FROM accelerometer_data")
    fun getAllAccelerometerDataFlow(): Flow<List<AccelerometerDataLocal>>

    @Insert
    suspend fun insertAllAccelerometerData(accelerometerDataLocal: List<AccelerometerDataLocal>)

    @Delete
    suspend fun deleteAccelerometerData(accelerometerDataLocal: AccelerometerDataLocal)

    @Delete
    suspend fun deleteAccelerometerDataList(accelerometerDataLocal: List<AccelerometerDataLocal>)

    @Query("DELETE FROM accelerometer_data")
    suspend fun deleteAllAccelerometerData()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertAccelerometerData(accelerometerData: AccelerometerDataLocal)

    @Query("SELECT * FROM gyroscope_data")
    fun getAllGyroscopeDataFlow(): Flow<List<GyroscopeDataLocal>>

    @Insert
    suspend fun insertAllGyroscopeData(gyroscopeDataLocal: List<GyroscopeDataLocal>)

    @Delete
    suspend fun deleteGyroscopeData(gyroscopeDataLocal: GyroscopeDataLocal)

    @Delete
    suspend fun deleteGyroscopeDataList(gyroscopeDataLocal: List<GyroscopeDataLocal>)

    @Query("DELETE FROM gyroscope_data")
    suspend fun deleteAllGyroscopeData()

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertGyroscopeData(gyroscopeDataLocal: GyroscopeDataLocal)
}
