package com.bureau.behavioralbiometrics.data.remote

import com.bureau.base.Environment
import com.bureau.behavioralbiometrics.data.remote.models.RegisterSubSessionRequestApiModel
import com.bureau.behavioralbiometrics.di.IS_DEV
import com.bureau.network.RetroFitInstance
import retrofit2.HttpException
import javax.inject.Inject
import javax.inject.Named

class BBRemoteDataSourceImpl @Inject constructor(environment: Environment, @Named(IS_DEV) isDev: Boolean) : BBRemoteDataSource {
    private val baseUrl = if (isDev) {
        BASE_URl_DEV
    } else {
        if (environment == Environment.ENV_PRODUCTION) {
            BASE_URl_PROD
        } else {
            BASE_URl_STG
        }
    }
    private val service: NetworkService by lazy {
        RetroFitInstance.getRetrofitInstance(baseUrl).create(
            NetworkService::class.java
        )
    }
    override suspend fun registerSubSession(requestApiModel: RegisterSubSessionRequestApiModel, credId: String): Boolean {
        return try {
            val result = service.registerSubSession(requestApiModel, credId)
            result.code() == 200
        } catch (ex: HttpException) {
            false
        } catch (ex: Throwable) {
            false
        }
    }

    companion object {
        private const val BASE_URl_STG = "https://api.overwatch.stg.bureau.id/"
        private const val BASE_URl_PROD = "https://api.overwatch.bureau.id/"
        private const val BASE_URl_DEV = "https://api.overwatch.dev.bureau.id/"
    }
}
