package com.bureau.behavioralbiometrics.di

import android.app.Application
import android.content.Context
import android.view.WindowManager
import androidx.room.Room
import com.bureau.base.coroutines.CoroutineContextProvider
import com.bureau.behavioralbiometrics.data.local.BBDatabase
import com.bureau.behavioralbiometrics.data.local.dao.BBDao
import com.bureau.behavioralbiometrics.frameratecollection.FPSConfig
import dagger.Module
import dagger.Provides
import kotlinx.coroutines.CoroutineScope
import javax.inject.Named

@Module(includes = [BBAbstractModule::class])
class BBModule {
    @Provides
    fun provideRoomDatabase(application: Application): BBDatabase = Room.databaseBuilder(
        application,
        BBDatabase::class.java,
        "behavioralbiometricsdb"
    ).build()

    @Provides
    fun provideBBDao(bbDatabase: BBDatabase): BBDao = bbDatabase.bbDao()

    @Provides
    @Named(COROUTINE_SCOPE_IO)
    fun provideCoroutineScopeIO(coroutineContextProvider: CoroutineContextProvider): CoroutineScope =
        CoroutineScope(coroutineContextProvider.io)

    @Provides
    @Named(COROUTINE_SCOPE_MAIN)
    fun provideCoroutineScopeMain(coroutineContextProvider: CoroutineContextProvider): CoroutineScope =
        CoroutineScope(coroutineContextProvider.main)

    @Provides
    fun provideFPSConfig(application: Application): FPSConfig {
        val display =
            (application.getSystemService(Context.WINDOW_SERVICE) as WindowManager).defaultDisplay
        val deviceRefreshRateInMs =
            1000f / display.refreshRate
        val refreshRate = display.refreshRate

        return FPSConfig(deviceRefreshRateInMs = deviceRefreshRateInMs, refreshRate = refreshRate)
    }
}
