package com.bureau.behavioralbiometrics.focuschangedata

import android.app.Activity
import android.view.View
import com.bureau.behavioralbiometrics.di.BBActivityScope
import com.bureau.behavioralbiometrics.di.COROUTINE_SCOPE_IO
import com.bureau.behavioralbiometrics.focuschangedata.models.FocusChangeDataLocal
import com.bureau.behavioralbiometrics.focuschangedata.models.FocusChangeType
import com.bureau.behavioralbiometrics.models.TargetElement
import com.bureau.behavioralbiometrics.utils.TimeUtils
import com.bureau.behavioralbiometrics.utils.ViewUtils
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.SharedFlow
import kotlinx.coroutines.launch
import javax.inject.Inject
import javax.inject.Named

@BBActivityScope
class FocusChangeDataDetector @Inject constructor(activity: Activity, @Named(COROUTINE_SCOPE_IO) private val coroutineScopeIO: CoroutineScope) {

    private val _focusChangeDataLocalFlow = MutableSharedFlow<FocusChangeDataLocal>()

    val focusChangeDataLocalFlow: SharedFlow<FocusChangeDataLocal>
        get() = _focusChangeDataLocalFlow
    init {
        activity.window.decorView.viewTreeObserver.addOnGlobalFocusChangeListener { oldFocus, newFocus ->
            emitFocusChangeData(oldFocus, FocusChangeType.FOCUS_LOST)
            emitFocusChangeData(newFocus, FocusChangeType.FOCUS_GAINED)
        }
    }

    private fun emitFocusChangeData(view: View?, focusChangeType: FocusChangeType) {
        view?.let {
            coroutineScopeIO.launch {
                _focusChangeDataLocalFlow.emit(
                    FocusChangeDataLocal(
                        timeStamp = TimeUtils.getTimeSinceLastRebootMillis(),
                        focusChangeType = focusChangeType,
                        targetElement = TargetElement(
                            ViewUtils.getId(it),
                            ViewUtils.getType(it)
                        )
                    )
                )
            }
        }
    }
}
