package com.bureau.behavioralbiometrics.frameratecollection

import java.util.concurrent.TimeUnit

object Calculation {
    fun getDroppedSet(fpsConfig: FPSConfig, dataSet: List<Long>): List<Int> {
        val droppedSet: MutableList<Int> = ArrayList()
        var start: Long = -1
        for (value in dataSet) {
            if (start == -1L) {
                start = value
                continue
            }
            val droppedCount = droppedCount(start, value, fpsConfig.deviceRefreshRateInMs)
            if (droppedCount > 0) {
                droppedSet.add(droppedCount)
            }
            start = value
        }
        return droppedSet
    }

    private fun droppedCount(start: Long, end: Long, devRefreshRate: Float): Int {
        var count = 0
        val diffNs = end - start
        val diffMs = TimeUnit.MILLISECONDS.convert(diffNs, TimeUnit.NANOSECONDS)
        val dev = Math.round(devRefreshRate).toLong()
        if (diffMs > dev) {
            val droppedCount = diffMs / dev
            count = droppedCount.toInt()
        }
        return count
    }

    fun calculateMetric(
        fpsConfig: FPSConfig,
        dataSet: List<Long>,
        droppedSet: List<Int>
    ): Long {
        val timeInNS = dataSet[dataSet.size - 1] - dataSet[0]
        val size = getNumberOfFramesInSet(timeInNS, fpsConfig)

        // metric
        // total dropped
        var dropped = 0
        for (k in droppedSet) {
            dropped += k
        }
        val multiplier = fpsConfig.refreshRate / size
        val answer = multiplier * (size - dropped)
        return Math.round(answer).toLong()
    }

    private fun getNumberOfFramesInSet(realSampleLengthNs: Long, fpsConfig: FPSConfig): Long {
        val realSampleLengthMs =
            TimeUnit.MILLISECONDS.convert(realSampleLengthNs, TimeUnit.NANOSECONDS).toFloat()
        val size = realSampleLengthMs / fpsConfig.deviceRefreshRateInMs
        return Math.round(size).toLong()
    }
}
