package com.bureau.behavioralbiometrics.frameratecollection

import android.view.Choreographer
import com.bureau.behavioralbiometrics.di.BBScope
import com.bureau.behavioralbiometrics.di.COROUTINE_SCOPE_IO
import com.bureau.behavioralbiometrics.frameratecollection.Calculation.calculateMetric
import com.bureau.behavioralbiometrics.frameratecollection.Calculation.getDroppedSet
import com.bureau.behavioralbiometrics.frameratecollection.models.FrameRateEventLocal
import com.bureau.behavioralbiometrics.utils.TimeUtils
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.SharedFlow
import kotlinx.coroutines.launch
import javax.inject.Inject
import javax.inject.Named

@BBScope
class FPSFrameCallback @Inject constructor(
    private val fpsConfig: FPSConfig,
    @Named(COROUTINE_SCOPE_IO) private val coroutineScopeIO: CoroutineScope
) : Choreographer.FrameCallback {
    private val dataSet: MutableList<Long> = ArrayList()
    private var enabled = true
    private var startSampleTimeInNs: Long = 0

    private val _frameRateEventLocalFlow = MutableSharedFlow<FrameRateEventLocal>()

    val frameRateEventLocalFlow: SharedFlow<FrameRateEventLocal>
        get() = _frameRateEventLocalFlow

    fun setEnabled(enabled: Boolean) {
        this.enabled = enabled
    }

    override fun doFrame(frameTimeNanos: Long) {
        if (!enabled) {
            return
        }
        if (startSampleTimeInNs == 0L) {
            startSampleTimeInNs = frameTimeNanos
        }
        if (isFinishedWithSample(frameTimeNanos)) {
            collectSampleAndSend(frameTimeNanos)
        }
        dataSet.add(frameTimeNanos)

        Choreographer.getInstance().postFrameCallback(this)
    }

    private fun collectSampleAndSend(frameTimeNanos: Long) {
        val dataSetCopy: MutableList<Long> = ArrayList()
        dataSetCopy.addAll(dataSet)
        val droppedSet = getDroppedSet(fpsConfig, dataSet)
        val answer = calculateMetric(fpsConfig, dataSet, droppedSet)
        coroutineScopeIO.launch {
            _frameRateEventLocalFlow.emit(FrameRateEventLocal(timeStamp = TimeUtils.getTimeSinceLastRebootMillis(), frameRate = answer.toFloat()))
        }
        dataSet.clear()

        startSampleTimeInNs = frameTimeNanos
    }

    private fun isFinishedWithSample(frameTimeNanos: Long): Boolean {
        return frameTimeNanos - startSampleTimeInNs > fpsConfig.sampleTimeInNs
    }
}
