package com.bureau.behavioralbiometrics.touchtypedata

import android.widget.AbsListView

/**
 * Created by mariotaku on 14/10/22.
 */
class ListScrollDistanceCalculator : AbsListView.OnScrollListener {
    private var mScrollDistanceListener: ScrollDistanceListener? = null
    private var mListScrollStarted = false
    private var mFirstVisibleItem = 0
    private var mFirstVisibleHeight = 0
    private var mFirstVisibleTop = 0
    private var mFirstVisibleBottom = 0
    private var totalScrollDistance = 0

    override fun onScrollStateChanged(view: AbsListView, scrollState: Int) {
        if (view.count == 0) return
        when (scrollState) {
            AbsListView.OnScrollListener.SCROLL_STATE_IDLE -> {
                mListScrollStarted = false
            }

            AbsListView.OnScrollListener.SCROLL_STATE_TOUCH_SCROLL -> {
                val firstChild = view.getChildAt(0)
                mFirstVisibleItem = view.firstVisiblePosition
                mFirstVisibleTop = firstChild.top
                mFirstVisibleBottom = firstChild.bottom
                mFirstVisibleHeight = firstChild.height
                mListScrollStarted = true
                totalScrollDistance = 0
            }
        }
    }

    override fun onScroll(
        view: AbsListView,
        firstVisibleItem: Int,
        visibleItemCount: Int,
        totalItemCount: Int
    ) {
        if (totalItemCount == 0 || !mListScrollStarted) return
        val firstChild = view.getChildAt(0)
        val firstVisibleTop = firstChild.top
        val firstVisibleBottom = firstChild.bottom
        val firstVisibleHeight = firstChild.height
        val delta: Int
        if (firstVisibleItem > mFirstVisibleItem) {
            mFirstVisibleTop += mFirstVisibleHeight
            delta = firstVisibleTop - mFirstVisibleTop
        } else if (firstVisibleItem < mFirstVisibleItem) {
            mFirstVisibleBottom -= mFirstVisibleHeight
            delta = firstVisibleBottom - mFirstVisibleBottom
        } else {
            delta = firstVisibleBottom - mFirstVisibleBottom
        }
        totalScrollDistance += delta
        mScrollDistanceListener?.onScrollDistanceChanged(delta, totalScrollDistance)
        mFirstVisibleTop = firstVisibleTop
        mFirstVisibleBottom = firstVisibleBottom
        mFirstVisibleHeight = firstVisibleHeight
        mFirstVisibleItem = firstVisibleItem
    }

    fun setScrollDistanceListener(listener: ScrollDistanceListener?) {
        mScrollDistanceListener = listener
    }

    interface ScrollDistanceListener {
        fun onScrollDistanceChanged(delta: Int, total: Int)
    }
}
