package com.bureau.behavioralbiometrics.touchtypedata.models

import android.view.MotionEvent
import androidx.room.Embedded
import androidx.room.Entity
import androidx.room.PrimaryKey
import com.bureau.behavioralbiometrics.data.remote.protoModels.closedRange
import com.bureau.behavioralbiometrics.data.remote.protoModels.offset2D
import com.bureau.behavioralbiometrics.data.remote.protoModels.pointerData
import com.bureau.behavioralbiometrics.models.TargetElement
import com.bureau.behavioralbiometrics.utils.ViewUtils

@Entity(tableName = "touch_type_data")
data class TouchTypeDataLocal(
    @PrimaryKey(autoGenerate = true) var id: Int = 0,
    val timeStamp: Long,
    val touchToolType: TouchToolType,
    @Embedded("offset2D_") val offset2D: Offset2D,
    val deviceId: Long,
    @Embedded("position2D_") val position2D: Position2D,
    val pressure: Double,
    @Embedded("pressureRange_") val pressureRange: ClosedRange,
    val radiusMajor: Double,
    val radiusMinor: Double,
    val orientation: Double,
    @Embedded("targetElement_") val targetElement: TargetElement,
    val gesture: Gesture
)

enum class TouchToolType(val toolType: Int) {
    STYLUS(MotionEvent.TOOL_TYPE_STYLUS),
    TOUCH(MotionEvent.TOOL_TYPE_FINGER),
    UNKNOWN(MotionEvent.TOOL_TYPE_UNKNOWN),
    MOUSE(MotionEvent.TOOL_TYPE_MOUSE),
    ERASER(MotionEvent.TOOL_TYPE_ERASER);

    companion object {
        fun getTouchToolTypeFromToolType(toolType: Int): TouchToolType {
            return when (toolType) {
                MotionEvent.TOOL_TYPE_STYLUS -> STYLUS
                MotionEvent.TOOL_TYPE_FINGER -> TOUCH
                MotionEvent.TOOL_TYPE_MOUSE -> MOUSE
                MotionEvent.TOOL_TYPE_ERASER -> ERASER
                else -> UNKNOWN
            }
        }
    }
}

data class Offset2D(val xOffset: Double, val yOffset: Double)

data class Position2D(val xPosition: Double, val yPosition: Double)

data class ClosedRange(val lowerBound: Double, val upperBound: Double)

enum class Gesture {
    SCROLL,
    SINGE_TAP,
    DOUBLE_TAP,
    FLING,
    LONG_PRESS
}

fun TouchTypeDataLocal.toPointerData() = pointerData {
    determinedAt = timeStamp
    toolType = touchToolType.name
    delta = offset2D {
        dx = offset2D.xOffset
        dy = offset2D.yOffset
    }
    device = deviceId
    position = offset2D {
        dx = position2D.xPosition
        dy = position2D.yPosition
    }
    pressure = this@toPointerData.pressure
    pressureRange = closedRange {
        lowerBound = this@toPointerData.pressureRange.lowerBound
        upperBound = this@toPointerData.pressureRange.upperBound
    }
    radiusMajor = this@toPointerData.radiusMajor
    radiusMinor = this@toPointerData.radiusMinor
    orientation = this@toPointerData.orientation
    target = ViewUtils.getTarget(targetElement)
}
